package cd.casic.ci.common.pipeline.enums;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.api.enums
 * @Project：ops-pro
 * @name：AgentStatus
 * @Date：2025/03/26 10:13
 * @Filename：AgentStatus
 * @description：Todo
 */
@Getter
@Slf4j
@AllArgsConstructor
public enum AgentStatus {
    UN_IMPORT(0), // 未导入，用户刚刚在界面上面生成链接
    UN_IMPORT_OK(1), // 未导入但是agent状态正常（这个时候还是不能用来当构建机）
    IMPORT_OK(2), // 用户已经在界面导入并且agent工作正常（构建机只有在这个状态才能正常工作）
    IMPORT_EXCEPTION(3), // agent异常
    DELETE(4); // 删除

    private final int status;

    @Override
    public String toString() {
        return String.valueOf(status);
    }

    public static AgentStatus fromStatus(int status) {
        for (AgentStatus agentStatus : values()) {
            if (agentStatus.getStatus() == status) {
                return agentStatus;
            }
        }
        log.error("Unknown agent status(" + status + ")");
        return AgentStatus.UN_IMPORT;
    }

    public static boolean isDelete(AgentStatus status) {
        return status == DELETE;
    }

    public static boolean isUnImport(AgentStatus status) {
        return status == UN_IMPORT;
    }

    public static boolean isImportException(AgentStatus status) {
        return status == IMPORT_EXCEPTION;
    }

    public static AgentStatus fromString(String status) {
        for (AgentStatus agentStatus : values()) {
            if (agentStatus.name().equals(status)) {
                return agentStatus;
            }
        }
        log.error("Unknown agent status(" + status + ")");
        return AgentStatus.UN_IMPORT;
    }
}
