package cd.casic.ci.common.pipeline.enums;

import cd.casic.framework.commons.exception.ServerException;
import com.fasterxml.jackson.annotation.JsonIgnore;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;
import lombok.extern.slf4j.Slf4j;

import java.net.URLEncoder;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.api.enums
 * @Project：ops-pro
 * @name：RepositoryConfig
 * @Date：2025/03/26 10:27
 * @Filename：RepositoryConfig
 * @description：Todo
 */
@Data
@Slf4j
@NoArgsConstructor
@AllArgsConstructor
public class RepositoryConfig {
    private String repositoryHashId;
    @Schema(title = "新版的git代码库名")
    private String repositoryName;
    @Schema(title = "新版的git插件的类型")
    private RepositoryType repositoryType;

    public RepositoryConfig(String repositoryHashId, String repositoryName, TriggerRepositoryType triggerRepositoryType, String selfRepoHashId) {
        this.repositoryHashId = triggerRepositoryType == TriggerRepositoryType.SELF ? selfRepoHashId : repositoryHashId;
        this.repositoryName = repositoryName;
        this.repositoryType = TriggerRepositoryType.toRepositoryType(triggerRepositoryType) != null ? TriggerRepositoryType.toRepositoryType(triggerRepositoryType) : RepositoryType.ID;
    }

    @JsonIgnore
    public String getRepositoryId() {
        switch (repositoryType) {
            case ID:
                if (repositoryHashId == null || repositoryHashId.isBlank()) {
                    throw new ServerException(-1000, "代码库HashId为空");
                }
                return repositoryHashId;
            case NAME:
                if (repositoryName == null || repositoryName.isBlank()) {
                    throw new ServerException(-1000, "代码库名为空");
                }
                return repositoryName;
            default:
                throw new ServerException(-1000, "未知的代码库类型");
        }
    }

    @JsonIgnore
    public String getURLEncodeRepositoryId() {
        try {
            return URLEncoder.encode(getRepositoryId(), "UTF-8");
        } catch (Exception e) {
            throw new ServerException(-1000, "编码失败: " + e.getMessage());
        }
    }

    @Override
    public String toString() {
        return "[repositoryHashId=" + repositoryHashId + ", repositoryName=" + repositoryName + ", repositoryType=" + repositoryType + "]";
    }
}
