package cd.casic.ci.common.pipeline.extend;

import cd.casic.ci.common.pipeline.Model;
import cd.casic.ci.common.pipeline.container.Container;
import cd.casic.ci.common.pipeline.dialect.IPipelineDialect;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.common.pipeline.pojo.element.atom.BeforeDeleteParam;
import cd.casic.ci.common.pipeline.pojo.setting.PipelineSetting;
import cd.casic.framework.commons.exception.ServerException;

import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.extend
 * @Project：ops-pro
 * @name：ModelCheckPlugin
 * @Date：2025/03/27 10:39
 * @Filename：ModelCheckPlugin
 * @description：Todo
 */
public interface ModelCheckPlugin {

    /**
     * 检查[model]编排的完整性，并返回[JobSize + ElementSize = MetaSize]所有元素数量
     *
     * @param userId          操作人
     * @param oauthUser       当前流水线权限代持人
     * @param pipelineDialect 流水线方言,只有新增/编辑流水线或模版时才需要传入
     * @throws ServerException 检查失败或异常
     */
    int checkModelIntegrity(
            Model model,
            String projectId,
            String userId,
            boolean isTemplate,
            String oauthUser,
            IPipelineDialect pipelineDialect,
            String pipelineId
    ) throws ServerException;

    /**
     * 检查[setting]配置的完整性
     *
     * @throws ServerException 检查失败或异常
     */
    void checkSettingIntegrity(PipelineSetting setting, String projectId) throws ServerException;

    /**
     * 清理Model--不删除里面的Element内的逻辑
     */
    void clearUpModel(Model model);

    /**
     * 在删除element前做的一些处理
     * 对比sourceModel，并清理model中与之不同的Element
     *
     * @param existModel  目标Model（要清理的Model)
     * @param sourceModel 源要比较的Model
     */
    void beforeDeleteElementInExistsModel(
            Model existModel,
            Model sourceModel,
            BeforeDeleteParam param
    );

    /**
     * 检查[container]下的[element]插件的超时配置是否合法。
     * 如果使用了变量，则从变量表[contextMap]进行替换，如果值不符合，则抛出异常[ErrorCodeException]
     */
    void checkElementTimeoutVar(Container container, Element element, Map<String, String> contextMap) throws ServerException;

    /**
     * 检查[container]下互斥组配置是否合法，
     * 如果使用了变量，则从变量表[contextMap]进行替换，如果值不符合，则抛出异常[ErrorCodeException]
     */
    void checkMutexGroup(Container container, Map<String, String> contextMap) throws ServerException;

    /**
     * 检查是否是[finallyStage]Stage的Job[container]下[JobControlOption]配置是否合法。
     * 如果使用了变量，则从变量表[contextMap]进行替换，如果值不符合，则抛出异常[ErrorCodeException]
     */
    void checkJobCondition(Container container, boolean finallyStage, Map<String, String> contextMap) throws ServerException;
}
