package cd.casic.ci.common.pipeline.pojo.element.trigger;

import cd.casic.ci.common.pipeline.enums.StartType;
import cd.casic.ci.common.pipeline.enums.TriggerRepositoryType;
import cd.casic.ci.common.pipeline.pojo.element.trigger.enums.CodeEventType;
import cd.casic.ci.common.pipeline.pojo.element.trigger.enums.PathFilterType;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.experimental.Accessors;

import java.util.List;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.element.trigger
 * @Project：ops-pro
 * @name：CodeGithubWebHookTriggerElement
 * @Date：2025/03/27 9:42
 * @Filename：CodeGithubWebHookTriggerElement
 * @description：Todo
 */
@Accessors(chain = true)
@Schema(title = "Github事件触发", description = CodeGithubWebHookTriggerElement.classType)
public class CodeGithubWebHookTriggerElement extends WebHookTriggerElement {
    public static final String classType = "codeGithubWebHookTrigger";
    public static final String MERGE_ACTION_OPEN = "open";
    public static final String MERGE_ACTION_CLOSE = "close";
    public static final String MERGE_ACTION_REOPEN = "reopen";
    public static final String MERGE_ACTION_PUSH_UPDATE = "push-update";
    public static final String MERGE_ACTION_MERGE = "merge";

    @Schema(title = "任务名称", required = true)
    private String name = "GitHub事件触发";

    @Schema(title = "id", required = false)
    private String id;

    @Schema(title = "状态", required = false)
    private String status;

    @Schema(title = "仓库ID", required = true)
    private String repositoryHashId;

    @Schema(title = "分支名称", required = false)
    private String branchName;

    @Schema(title = "用于排除的分支名称", required = false)
    private String excludeBranchName;

    @Schema(title = "路径过滤类型", required = true)
    private PathFilterType pathFilterType = PathFilterType.NamePrefixFilter;

    @Schema(title = "用于包含的路径", required = false)
    private String includePaths;

    @Schema(title = "用于排除的路径", required = false)
    private String excludePaths;

    @Schema(title = "用户白名单", required = false)
    private List<String> includeUsers;

    @Schema(title = "用于排除的user id", required = false)
    private String excludeUsers;

    @Schema(title = "tag名称", required = false)
    private String tagName;

    @Schema(title = "用于排除的tag名称", required = false)
    private String excludeTagName;

    @Schema(title = "tag从哪条分支创建", required = false)
    private String fromBranches;

    @Schema(title = "用于排除的源分支名称", required = false)
    private String excludeSourceBranchName;

    @Schema(title = "用于包含的源分支名称", required = false)
    private String includeSourceBranchName;

    @Schema(title = "webhook队列", required = false)
    private Boolean webhookQueue = false;

    @Schema(title = "code review 类型", required = false)
    private List<String> includeCrTypes;

    @Schema(title = "是否启用回写")
    private Boolean enableCheck = true;

    @Schema(title = "push事件action")
    private List<String> includePushAction;

    @Schema(title = "是否启用第三方过滤")
    private Boolean enableThirdFilter = false;

    @Schema(title = "事件类型", required = false)
    private CodeEventType eventType;

    @Schema(title = "新版的github原子的类型")
    private TriggerRepositoryType repositoryType;

    @Schema(title = "新版的github代码库名")
    private String repositoryName;

    @Schema(title = "code review 状态", required = false)
    private List<String> includeCrState;

    @Schema(title = "code note comment", required = false)
    private String includeNoteComment;

    @Schema(title = "code note 类型", required = false)
    private List<String> includeNoteTypes;

    @Schema(title = "issue事件action")
    private List<String> includeIssueAction;

    @Schema(title = "pull request事件action")
    private List<String> includeMrAction = List.of(MERGE_ACTION_OPEN, MERGE_ACTION_REOPEN, MERGE_ACTION_PUSH_UPDATE);

    public CodeGithubWebHookTriggerElement(String name, String id, String status) {
        super(name, id, status);
    }

    @Override
    public String getClassType() {
        return classType;
    }

    @Override
    public String findFirstTaskIdByStartType(StartType startType) {
        return StartType.WEB_HOOK.name().equals(startType.name()) ? this.id : super.findFirstTaskIdByStartType(startType);
    }
}
