package cd.casic.ci.log.api.impl;


import cd.casic.ci.log.api.ServiceLogPrintResource;
import cd.casic.framework.commons.exception.ErrorCode;
import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.ci.log.dal.pojo.enums.LogStorageMode;
import cd.casic.ci.log.log.LogOriginEvent;
import cd.casic.ci.log.dal.pojo.message.LogMessage;
import cd.casic.ci.log.service.BuildLogPrintService;
import cd.casic.ci.log.service.IndexService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RestController;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@RestController
public class ServiceLogPrintResourceImpl implements ServiceLogPrintResource {

    @Resource
    private IndexService indexService;
    @Resource
    private BuildLogPrintService buildLogPrintService;

    private static Logger logger = LoggerFactory.getLogger(ServiceLogPrintResourceImpl.class);

    @Override
    public CommonResult<Boolean> addLogStatus(String buildId, String tag, String subTag, String jobId, int executeCount) {
        // 判断 buildId 是否为空
        if (buildId.isEmpty()) {
            logger.warn("Invalid build ID");
            return CommonResult.error(new ErrorCode(1000, "Invalid build ID"));
        }
        // 通过一次获取 创建记录以及缓存
        String index = indexService.getIndexName(buildId);
        logger.info("Start to print log to index: " + index);
//        buildLogPrintService.dispatchEvent(
//                new LogStatusEvent(buildId, false, tag, subTag, jobId, executeCount)
//        );
        return CommonResult.success(true);
    }

    @Override
    public CommonResult<Boolean> updateLogStatus(String buildId, boolean finished, String tag,
                                                 String subTag, String jobId, int executeCount, LogStorageMode logStorageMode) {
        if (Strings.isBlank(buildId)) {
            logger.warn("Invalid build ID：" + buildId);
            return CommonResult.error(new ErrorCode(1000, "Invalid build ID"));
        }
//        buildLogPrintService.dispatchEvent(
//                new LogStatusEvent(buildId,finished,tag,subTag,jobId,executeCount,logStorageMode)
//        );
        return CommonResult.success(true);
    }

    @Override
    public CommonResult<Boolean> addLogLine(String buildId, LogMessage logMessage) {
        if (Strings.isBlank(buildId)) {
            logger.warn("Invalid build ID" + buildId);
            return CommonResult.error(new ErrorCode(1000, "Invalid build ID"));
        }
        List<LogMessage> messages = new ArrayList<>();
        messages.add(logMessage);
        return buildLogPrintService.asyncDispatchEvent(new LogOriginEvent(buildId, messages));
    }

    @Override
    public CommonResult<Boolean> addLogMultiLine(String buildId, List<LogMessage> logMessages) {
        if (Strings.isBlank(buildId)) {
            logger.warn("Invalid build ID" + buildId);
            return CommonResult.error(new ErrorCode(1000, "Invalid build ID"));
        }
        buildLogPrintService.asyncDispatchEvent(new LogOriginEvent(buildId, logMessages));
        return CommonResult.success(true);
    }


}
