/* * Tencent is pleased to support the open source community by making BK-CI 蓝鲸持续集成平台 available. * * Copyright (C) 2019 THL A29 Limited, a Tencent company.  All rights reserved. * * BK-CI 蓝鲸持续集成平台 is licensed under the MIT license. * * A copy of the MIT License is included in this file. * * * Terms of the MIT License: * --------------------------------------------------- * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to * permit persons to whom the Software is furnished to do so, subject to the following conditions: * * The above copyright notice and this permission notice shall be included in all copies or substantial portions of * the Software. * * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */
package cd.casic.ci.process.process.dal.record;

import cd.casic.ci.common.pipeline.enums.BuildRecordTimeStamp;
import cd.casic.ci.common.pipeline.enums.BuildStatus;
import cd.casic.ci.common.pipeline.pojo.time.BuildTimestampType;
import cd.casic.ci.process.api.process.pojo.pipeline.record.BuildRecordTask;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 构建记录任务 Dao 接口
 */
@Mapper
@Repository
public interface BuildRecordTaskDao {
    /**
     * 批量保存构建记录任务     *     * @param records 构建记录任务列表
     */
    void batchSave(@Param("records") List<BuildRecordTask> records);

    /**
     * 更新构建记录     *     * @param projectId 项目ID     * @param pipelineId 流水线ID     * @param buildId 构建ID     * @param taskId 任务ID     * @param executeCount 执行次数     * @param taskVar 任务变量     * @param buildStatus 构建状�?     * @param startTime 开始时�?     * @param endTime 结束时间     * @param timestamps 时间�?
     */
    void updateRecord(@Param("projectId") String projectId, @Param("pipelineId") String pipelineId, @Param("buildId") String buildId, @Param("taskId") String taskId, @Param("executeCount") int executeCount, @Param("taskVar") Map<String, Object> taskVar, @Param("buildStatus") BuildStatus buildStatus, @Param("startTime") LocalDateTime startTime, @Param("endTime") LocalDateTime endTime, @Param("timestamps") Map<BuildTimestampType, BuildRecordTimeStamp> timestamps);

    /**
     * 更新构建记录状�?     *     * @param projectId 项目ID     * @param pipelineId 流水线ID     * @param buildId 构建ID     * @param executeCount 执行次数     * @param buildStatus 构建状�?     * @param stageId 阶段ID     * @param containerId 容器ID
     */
    void updateRecordStatus(@Param("projectId") String projectId, @Param("pipelineId") String pipelineId, @Param("buildId") String buildId, @Param("executeCount") int executeCount, @Param("buildStatus") BuildStatus buildStatus, @Param("stageId") String stageId, @Param("containerId") String containerId);

    /**
     * 获取构建记录列表     *     * @param projectId 项目ID     * @param pipelineId 流水线ID     * @param buildId 构建ID     * @param executeCount 执行次数     * @param containerId 容器ID     * @param buildStatusSet 构建状态集�?     * @return 构建记录任务列表
     */
    List<BuildRecordTask> getRecords(@Param("projectId") String projectId, @Param("pipelineId") String pipelineId, @Param("buildId") String buildId, @Param("executeCount") int executeCount, @Param("containerId") String containerId, @Param("buildStatusSet") Set<BuildStatus> buildStatusSet);

    /**
     * 获取最新的普通构建记录列�?     *     * @param projectId 项目ID     * @param buildId 构建ID     * @param executeCount 执行次数     * @param matrixContainerIds 矩阵容器ID列表     * @param stageId 阶段ID     * @return 构建记录任务列表
     */
    List<BuildRecordTask> getLatestNormalRecords(@Param("projectId") String projectId, @Param("buildId") String buildId, @Param("executeCount") int executeCount, @Param("matrixContainerIds") List<String> matrixContainerIds, @Param("stageId") String stageId);

    /**
     * 获取最新的矩阵构建记录列表     *     * @param projectId 项目ID     * @param buildId 构建ID     * @param executeCount 执行次数     * @param matrixContainerIds 矩阵容器ID列表     * @return 构建记录任务列表
     */
    List<BuildRecordTask> getLatestMatrixRecords(@Param("projectId") String projectId, @Param("buildId") String buildId, @Param("executeCount") int executeCount, @Param("matrixContainerIds") List<String> matrixContainerIds);

    /**
     * 获取构建记录     *     * @param projectId 项目ID     * @param pipelineId 流水线ID     * @param buildId 构建ID     * @param taskId 任务ID     * @param executeCount 执行次数     * @return 构建记录任务
     */
    BuildRecordTask getRecord(@Param("projectId") String projectId, @Param("pipelineId") String pipelineId, @Param("buildId") String buildId, @Param("taskId") String taskId, @Param("executeCount") int executeCount);

    /**
     * 更新异步状�?     *     * @param projectId 项目ID     * @param pipelineId 流水线ID     * @param buildId 构建ID     * @param taskId 任务ID     * @param executeCount 执行次数     * @param asyncStatus 异步状�?
     */
    void updateAsyncStatus(@Param("projectId") String projectId, @Param("pipelineId") String pipelineId, @Param("buildId") String buildId, @Param("taskId") String taskId, @Param("executeCount") int executeCount, @Param("asyncStatus") String asyncStatus);

    /**
     * 重试时清空结束时�?     *     * @param projectId 项目ID     * @param pipelineId 流水线ID     * @param buildId 构建ID     * @param taskId 任务ID     * @param executeCount 执行次数
     */
    void flushEndTimeWhenRetry(@Param("projectId") String projectId, @Param("pipelineId") String pipelineId, @Param("buildId") String buildId, @Param("taskId") String taskId, @Param("executeCount") int executeCount);
}
