/* * Tencent is pleased to support the open source community by making BK-CI 蓝鲸持续集成平台 available. * * Copyright (C) 2019 THL A29 Limited, a Tencent company.  All rights reserved. * * BK-CI 蓝鲸持续集成平台 is licensed under the MIT license. * * A copy of the MIT License is included in this file. * * * Terms of the MIT License: * --------------------------------------------------- * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to * permit persons to whom the Software is furnished to do so, subject to the following conditions: * * The above copyright notice and this permission notice shall be included in all copies or substantial portions of * the Software. * * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */
package cd.casic.ci.process.process.dal.report;

import cd.casic.ci.process.api.process.pojo.Report;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 报告 Dao 接口
 */
@Mapper
@Repository
public interface ReportDao {
    /**
     * 获取原子信息     *     * @param buildId 构建ID     * @param taskId 任务ID     * @return 原子信息
     */
    Map<String, String> getAtomInfo(@Param("buildId") String buildId, @Param("taskId") String taskId);

    /**
     * 创建报告     *     * @param projectId 项目ID     * @param pipelineId 流水线ID     * @param buildId 构建ID     * @param elementId 元素ID     * @param indexFile 索引文件     * @param name 名称     * @param type 类型     * @param atomCode 原子代码     * @param taskName 任务名称     * @param id 记录ID     * @return 记录ID
     */
    long createReport(@Param("projectId") String projectId, @Param("pipelineId") String pipelineId, @Param("buildId") String buildId, @Param("elementId") String elementId, @Param("indexFile") String indexFile, @Param("name") String name, @Param("type") String type, @Param("atomCode") String atomCode, @Param("taskName") String taskName, @Param("id") Long id);

    /**
     * 获取报告     *     * @param projectId 项目ID     * @param pipelineId 流水线ID     * @param buildId 构建ID     * @param elementId 元素ID     * @return 报告
     */
    Report getReport(@Param("projectId") String projectId, @Param("pipelineId") String pipelineId, @Param("buildId") String buildId, @Param("elementId") String elementId);

    /**
     * 获取报告列表     *     * @param projectId 项目ID     * @param pipelineId 流水线ID     * @param buildId 构建ID     * @return 报告列表
     */
    List<Report> getReportList(@Param("projectId") String projectId, @Param("pipelineId") String pipelineId, @Param("buildId") String buildId);

    /**
     * 删除报告     *     * @param projectId 项目ID     * @param pipelineId 流水线ID     * @param buildId 构建ID     * @param elementId 元素ID     * @return 影响的行�?
     */
    int deleteReport(@Param("projectId") String projectId, @Param("pipelineId") String pipelineId, @Param("buildId") String buildId, @Param("elementId") String elementId);
}
