package cd.casic.ci.project.service.impl;


import cd.casic.ci.common.pipeline.constant.BaseConstant;
import cd.casic.ci.project.constant.ProjectConstants;
import cd.casic.ci.project.dal.dao.ProjectMapper;
import cd.casic.ci.project.dal.dto.Project;
import cd.casic.ci.project.dal.pojo.*;
import cd.casic.ci.project.dal.pojo.user.UserDeptDetail;
import cd.casic.ci.project.enums.ProjectApproveStatus;
import cd.casic.ci.project.enums.ProjectAuthSecrecyStatus;
import cd.casic.ci.project.enums.ProjectChannelCode;
import cd.casic.ci.project.enums.ProjectValidateType;
import cd.casic.ci.project.service.ProjectExtService;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.util.json.JsonUtils;
import cd.casic.framework.redis.core.RedisTemplateUtils;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;


@Service("projectService")
@Slf4j
public class ProjectServiceImpl {

    private final String ENGLISH_NAME_PATTERN = "[a-z][a-zA-Z0-9-]+";
    private final String ALL_MEMBERS = "*";
    private final String ALL_MEMBERS_NAME = "allMembersName";
    private final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    @Resource
    private ProjectExtService projectExtService;
    @Resource
    private ProjectMapper projectMapper;
    @Resource
    private RedisTemplateUtils redisTemplateUtils;

    public String create(Long userId, ProjectCreateInfo projectCreateInfo, String accessToken,
                         ProjectCreateExtInfo createExtInfo, String defaultProjectId, ProjectChannelCode projectChannelCode) {

        log.info("create project| {} | {} | {} | {}", userId, accessToken, createExtInfo, projectCreateInfo);
        if (createExtInfo.getNeedValidate()) {
            validate(ProjectValidateType.PROJECT_NAME, projectCreateInfo.getProjectName(), null);
            validate(ProjectValidateType.ENGLISH_NAME, projectCreateInfo.getEnglishName(), null);
        }
        UserDeptDetail userDeptDetail = getDeptInfo(userId);
        String projectId = defaultProjectId;
        List<SubjectScopeInfo> subjectScopes = projectCreateInfo.getSubjectScopes();
        if (subjectScopes == null || subjectScopes.isEmpty()) {
            subjectScopes = Collections.singletonList(new SubjectScopeInfo(ALL_MEMBERS, getAllMembersName(), ALL_MEMBERS, null, null));
        }
        //TODO 待完善 先直接得到结果
//        boolean needApproval = projectPermissionService.needApproval(createExtInfo.getNeedApproval());
        boolean needApproval = false;
        int approvalStatus = needApproval ? ProjectApproveStatus.CREATE_PENDING.getStatus() :
                ProjectApproveStatus.APPROVED.getStatus();

        //这一块bk-ci未具体实现
//        try {
//            if (createExtInfo.getNeedAuth()) {
//                AuthProjectCreateInfo authProjectCreateInfo = new AuthProjectCreateInfo(userId, accessToken,
//                    userDeptDetail, subjectScopes, projectCreateInfo, approvalStatus);
//                // 注册项目到权限中心(bk-ci未具体实现，只是一个空方法)
//                projectId = projectPermissionService.createResources(
//                    new ResourceRegisterInfo(projectCreateInfo.getEnglishName(),
//                        projectCreateInfo.getProjectName()), authProjectCreateInfo);
//            }
//        } catch (PermissionForbiddenException e) {
//            throw e;
//        } catch (Exception e) {
//            logger.warn("Failed to create project in permission center: {} | {}", projectCreateInfo, e.getMessage());
//            throw new OperationException(I18nUtil.getCodeLanMessage(ProjectMessageCode.PEM_CREATE_FAIL));
//        }
        if (StringUtils.isEmpty(projectId)) {
            projectId = IdUtil.simpleUUID();
        }
        try {
            String subjectScopesStr = JsonUtils.toJsonString(subjectScopes);
            String logoAddress = projectCreateInfo.getLogoAddress();

            create(userId, logoAddress, projectCreateInfo, userDeptDetail, projectId,
                    projectChannelCode, approvalStatus, subjectScopesStr);
            if (!needApproval) {
                projectExtService.createExtProjectInfo(userId, projectId, accessToken, projectCreateInfo,
                        createExtInfo, logoAddress);
            }
            // 为项目分配数据源
//            shardingRoutingRuleAssignService.assignShardingRoutingRule(projectChannelCode,
//                projectCreateInfo.getEnglishName(), Arrays.asList(SystemModuleEnum.PROCESS, SystemModuleEnum.METRICS));
            if (projectCreateInfo.isSecrecy()) {
                redisTemplateUtils.set(ProjectConstants.SECRECY_PROJECT_REDIS_KEY, projectCreateInfo.getEnglishName());
            }
            updateProjectRouterTag(projectCreateInfo.getEnglishName());
        } catch (Exception e) {
            log.warn("Duplicate project: {}", projectCreateInfo, e);
            if (createExtInfo.getNeedAuth()) {
                // todo 待确定，切换v3-RBAC后，是否需要做其他操作

//                deleteAuth(projectId, accessToken);
            }
            throw new ServiceException(-1000, "项目名或英文名重复");
        } catch (Throwable e) {
            log.warn("Fail to create the project: {}", projectCreateInfo, e);
            if (createExtInfo.getNeedAuth()) {
//                deleteAuth(projectId, accessToken);
            }
            throw e;
        }
        return projectId;
    }

    private void updateProjectRouterTag(String englishName) {
    }

    private String getAllMembersName() {
        //TODO
//        return I18nUtil.getCodeLanMessage(ALL_MEMBERS_NAME);
        return "allMembersName";
    }

    private UserDeptDetail getDeptInfo(Long userId) {
        UserDeptDetail detail = new UserDeptDetail();
        detail.setUserId(userId);
        detail.setBgId("1");
        detail.setCenterId("1");
        detail.setDeptId("1");

        return detail;
    }


    public void validate(ProjectValidateType validateType, String name, String projectId) {
//        if (StringUtils.isBlank(name)) {
//            throw new ServiceException(-1000, "项目名称不能为空");
//        }
//        if (validateType == ProjectValidateType.PROJECT_NAME) {
//            if (name.isEmpty() || name.length() > ProjectConstants.NAME_MAX_LENGTH) {
//                throw new ServiceException(-1000, "项目名至多1-64个字符");
//            }
//            LambdaQueryWrapper<Project> wrapper = Wrappers.<Project>lambdaQuery()
//                    .eq(Project::getProjectName, name);
//            if (StringUtils.isNotBlank(projectId)) {
//                wrapper.ne(Project::getProjectId, projectId);
//            }
//
//            if (this.exist(wrapper)) {
//                throw new ServiceException(-1000, "项目名或英文名重复");
//            }
//        } else if (validateType == ProjectValidateType.ENGLISH_NAME) {
//            if (name.length() < ProjectConstants.NAME_MIN_LENGTH || name.length() > ProjectConstants.NAME_MAX_LENGTH) {
//                throw new ServiceException(-1000, "英文名长度在3-64个字符");
//            }
//            if (!Pattern.matches(ENGLISH_NAME_PATTERN, name)) {
//                log.warn("Project English Name({}) is not match", name);
//                throw new ServiceException(-1000, "英文名是字符+数字组成，并以小写字母开头");
//            }
//
//            LambdaQueryWrapper<Project> queryWrapper = Wrappers.<Project>lambdaQuery()
//                    .eq(Project::getEnglishName, name);
//            if (StringUtils.isNotBlank(projectId)) {
//                queryWrapper.ne(Project::getProjectId, projectId);
//            }
//            if (this.exist(queryWrapper)) {
//                throw new ServiceException(-1000, "英文名已经存在");
//            }
//        }
    }

    public ProjectVO getByProjectId(String projectId) {
        Project one = projectMapper.selectOne(Project::getProjectId, projectId, Project::getDeleted, BaseConstant.NOT_DELETED);
        ProjectVO convert = Convert.convert(ProjectVO.class, one);
        convert.setProjectCode(one.getEnglishName() == null ? "" : one.getEnglishName());
        convert.setGray("gray".equals(one.getRouterTag()));
        convert.setChannelCode(one.getChannel());
        convert.setUpdator(one.getUpdater());
        convert.setCreatedAt(DateUtil.format(one.getCreateTime(), YYYY_MM_DD_HH_MM_SS));
        convert.setUpdatedAt(DateUtil.format(one.getUpdateTime(), YYYY_MM_DD_HH_MM_SS));
        convert.setApprovalTime(DateUtil.format(one.getApprovalTime(), YYYY_MM_DD_HH_MM_SS));
        convert.setProperties(JsonUtils.parseObject(one.getProperties(), ProjectProperties.class));
        convert.setSubjectScopes(JsonUtils.parseArray(one.getSubjectScopes(), SubjectScopeInfo.class));
        return convert;
    }

    private void create(Long userId, String logoAddress, ProjectCreateInfo projectCreateInfo, UserDeptDetail userDeptDetail,
                        String projectId, ProjectChannelCode channelCode, Integer approvalStatus, String subjectScopesStr) {
        Project project = Convert.convert(Project.class, projectCreateInfo);
        project.setProjectId(projectId);
        project.setCreator(userId.toString());
        project.setApprovalStatus(approvalStatus);
        project.setLogoAddr(logoAddress == null ? "" : logoAddress);
        project.setCreatorBgName(projectCreateInfo.getBgName());
        project.setCreatorDeptName(projectCreateInfo.getBgName());
        project.setCreatorCenterName(projectCreateInfo.getCenterName());
        project.setChannel(channelCode != null ? channelCode.name() : null);
        project.setEnabled(true);
        project.setProperties(projectCreateInfo.getProperties() != null ? JsonUtils.toJsonString(projectCreateInfo.getProperties()) : null);
        project.setSubjectScopes(subjectScopesStr);
        project.setAuthSecrecy(projectCreateInfo.getAuthSecrecy() == null ? ProjectAuthSecrecyStatus.PUBLIC.getValue() : projectCreateInfo.getAuthSecrecy());
        projectMapper.insert(project);
    }
}
