package cd.casic.ci.log.scm.service;


import cd.casic.ci.log.scm.dal.pojo.*;
import cd.casic.ci.log.scm.service.code.CodeRepositoryService;
import cd.casic.ci.log.scm.enums.ScmType;

import java.util.concurrent.ConcurrentHashMap;

/**
 * @author by mianbin
 * @Classname CodeRepositoryServiceRegistrar
 * @Description TODO
 * @Date 2025/3/21 10:22
 */
public class CodeRepositoryServiceRegistrar {
    private static ConcurrentHashMap<String, CodeRepositoryService<?>> repositoryServiceMap = new ConcurrentHashMap<>();

    public static CodeRepositoryService<?> getServiceByScmType(String scmType) {
        CodeRepositoryService<? extends Repository> repositoryService;
        ScmType type = ScmType.valueOf(scmType);
        if (type == ScmType.CODE_SVN) {
            repositoryService = getService(CodeSvnRepository.class);
        }
        else if (type == ScmType.CODE_GIT) {
            repositoryService = getService(CodeGitRepository.class);
        }
        else if (type == ScmType.CODE_TGIT) {
            repositoryService = getService(CodeTGitRepository.class);
        }
        else if (type == ScmType.CODE_GITLAB) {
            repositoryService = getService(CodeGitlabRepository.class);
        }
        else if (type == ScmType.GITHUB) {
            repositoryService = getService(GithubRepository.class);
        }
        else if (type == ScmType.CODE_P4) {
            repositoryService = getService(CodeP4Repository.class);
        }
        else {
            throw new IllegalArgumentException("Unknown repository type");
        }
        return repositoryService;
    }

    /**
     * 根据类型读取指定[CodeRepositoryService]代码库业务处理器
     */
    private static  <T> CodeRepositoryService<T> getService(Class<? extends T> type) {
        if (type != null) {
            CodeRepositoryService<?> repositoryService = repositoryServiceMap.get(type.getName());
            if (repositoryService == null) {
                throw new IllegalArgumentException(type.getName() + "handler is not found");
            }
            return (CodeRepositoryService<T>) repositoryService;
        } else {
            throw new IllegalArgumentException("Unknown repository type");
        }
    }
}
