package cd.casic.ci.log.scm.service.scm;


import cd.casic.ci.log.scm.config.GitConfig;
import cd.casic.ci.log.scm.config.SVNConfig;
import cd.casic.ci.log.scm.dal.pojo.RevisionInfo;
import cd.casic.ci.log.scm.dal.pojo.ScmOauthFactory;
import cd.casic.ci.log.scm.enums.CodeSvnRegion;
import cd.casic.ci.log.scm.enums.ScmType;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;


@Service
@Slf4j
public class ScmOauthServiceImpl implements IScmOauthService {

    @Resource
    private GitConfig gitConfig;

    @Resource
    private SVNConfig svnConfig;

    @Override
    public RevisionInfo getLatestRevision(String projectName, String url, ScmType type, String branchName, String privateKey, String passPhrase, String token, CodeSvnRegion region, String userName) {
        log.info("[{}|{}|{}|{}|{}] Start to get the latest oauth revision", projectName, url, type, branchName, userName);
        long startEpoch = System.currentTimeMillis();
        try {
            return ScmOauthFactory.getScm(projectName, url, type, branchName, privateKey, passPhrase, token, region, userName, null).getLatestRevision();
        } finally {
            log.info("It took {}ms to get the latest revision", System.currentTimeMillis() - startEpoch);
        }
    }

    @Override
    public void addWebhook(String projectName, String url, ScmType type, String privateKey, String passPhrase, String token, CodeSvnRegion region, String userName, String event) {
        log.info("[{}|{}|{}|{}] Start to add web hook", projectName, url, type, userName);
        long startEpoch = System.currentTimeMillis();
        try {
            String hookUrl;
            if (type == ScmType.CODE_GIT) {
                hookUrl = gitConfig.gitHookUrl;
            } else if (type == ScmType.CODE_GITLAB) {
                hookUrl = gitConfig.gitlabHookUrl;
            } else if (type == ScmType.CODE_SVN) {
                hookUrl = svnConfig.svnHookUrl;
            } else if (type == ScmType.CODE_TGIT) {
                hookUrl = gitConfig.tGitHookUrl;
            } else {
                throw new IllegalArgumentException("Unknown repository type (" + type + ") when add webhook");
            }
            ScmOauthFactory.getScm(projectName, url, type, null, privateKey, passPhrase, token, region, userName, event)
                    .addWebhook(hookUrl);
        } finally {
            log.info("It took {}ms to add web hook", System.currentTimeMillis() - startEpoch);
        }
    }
}
