package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.visible.StoreVisibleDeptReq;
import cd.casic.pipeline.store.pojo.common.visible.StoreVisibleDeptResp;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.boot.context.properties.bind.DefaultValue;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;

@Tag(name = "USER_STORE_VISIBLE", description = "研发商店-可见范围")
@RestController
@RequestMapping("/user/store/visibilities")
public interface UserStoreVisibleDeptResource {

    @Operation(summary = "设置组件可见范围")
    @PostMapping("/add")
    CommonResult<Boolean> addVisibleDept(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件可见范围请求报文体", required = true)
            StoreVisibleDeptReq storeVisibleDeptReq
    );

    @Operation(summary = "查看组件可见范围")
    @GetMapping("/types/{storeType}/codes/{storeCode}/get")
    CommonResult<StoreVisibleDeptResp> getVisibleDept(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "机构状态集合，用\",\"分隔进行拼接（如APPROVING,APPROVED,REJECT）", required = false)
            @RequestParam("deptStatusInfos")
            @DefaultValue("APPROVED")
            String deptStatusInfos
    );

    @Operation(summary = "删除组件可见范围")
    @DeleteMapping("/types/{storeType}/codes/{storeCode}/delete")
    CommonResult<Boolean> deleteVisibleDept(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "机构Id集合，用\",\"分隔进行拼接（如1,2,3）", required = true)
            @RequestParam("deptIds")
            String deptIds
    );
}