package cd.casic.pipeline.store.api.image;

import cd.casic.ci.common.pipeline.type.docker.ImageType;
import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.pipeline.store.pojo.common.version.VersionInfo;
import cd.casic.pipeline.store.pojo.image.enums.ImageRDTypeEnum;
import cd.casic.pipeline.store.pojo.image.enums.MarketImageSortTypeEnum;
import cd.casic.pipeline.store.pojo.image.request.ImageBaseInfoUpdateRequest;
import cd.casic.pipeline.store.pojo.image.response.ImageDetail;
import cd.casic.pipeline.store.pojo.image.response.MarketImageMain;
import cd.casic.pipeline.store.pojo.image.response.MarketImageResp;
import cd.casic.pipeline.store.pojo.image.response.MyImage;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;
import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID_DEFAULT_VALUE;

@Tag(name = "USER_MARKET_IMAGE", description = "研发商店-镜像")
@RestController
@RequestMapping("/user/market")
public interface UserMarketImageResource {

    @Operation(summary = "镜像市场首页")
    @GetMapping("/image/list/main")
    CommonResult<List<MarketImageMain>> mainPageList(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页大小", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "镜像市场搜索镜像")
    @GetMapping("/image/list")
    CommonResult<MarketImageResp> searchImage(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "搜索关键字", required = false)
            @RequestParam("keyword")
            String keyword,
            @Parameter(description = "镜像来源", required = false)
            @RequestParam("imageSourceType")
            ImageType imageSourceType,
            @Parameter(description = "镜像分类编码", required = false)
            @RequestParam("classifyCode")
            String classifyCode,
            @Parameter(description = "应用范畴", required = false)
            @RequestParam("categoryCode")
            String categoryCode,
            @Parameter(description = "镜像研发来源", required = false)
            @RequestParam("rdType")
            ImageRDTypeEnum rdType,
            @Parameter(description = "镜像标签代码", required = false)
            @RequestParam("labelCode")
            String labelCode,
            @Parameter(description = "镜像评分", required = false)
            @RequestParam("score")
            Integer score,
            @Parameter(description = "镜像排序字段", required = false)
            @RequestParam("sortType")
            MarketImageSortTypeEnum sortType,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页大小", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "根据ID查询镜像详情")
    @GetMapping("/image/imageIds/{imageId}")
    CommonResult<ImageDetail> getImageDetailById(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "镜像Id", required = true)
            @PathVariable("imageId")
            String imageId
    );

    @Operation(summary = "根据code查询镜像详情")
    @GetMapping("/image/imageCodes/{imageCode}")
    CommonResult<ImageDetail> getImageDetailByCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "镜像标识", required = true)
            @PathVariable("imageCode")
            String imageCode
    );

    @Operation(summary = "查询镜像版本列表")
    @GetMapping("/image/imageCodes/{imageCode}/version/list")
    PageResult<ImageDetail> getImageVersionListByCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "镜像标识", required = true)
            @PathVariable("imageCode")
            String imageCode,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页大小", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "我的镜像列表")
    @GetMapping("/desk/image/list")
    PageResult<MyImage> getMyImageList(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "镜像名称", required = false)
            @RequestParam("imageName")
            String imageName,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页大小", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "删除镜像")
    @DeleteMapping("/image/imageCodes/{imageCode}")
    CommonResult<Boolean> delete(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "镜像Code", required = true)
            @PathVariable("imageCode")
            String imageCode
    );

    @Operation(summary = "更新流水线镜像信息")
    @PutMapping("/baseInfo/images/{imageCode}")
    CommonResult<Boolean> updateImageBaseInfo(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "镜像代码 ", required = true)
            @PathVariable("imageCode")
            String imageCode,
            @Parameter(description = "镜像基本信息修改请求报文体", required = true)
            ImageBaseInfoUpdateRequest imageBaseInfoUpdateRequest
    );

    @Operation(summary = "根据镜像代码获取对应的版本列表信息")
    @GetMapping("/projectCodes/{projectCode}/imageCodes/{imageCode}/version/list")
    CommonResult<List<VersionInfo>> getPipelineImageVersions(
            @Parameter(description = "项目代码", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "镜像代码", required = true)
            @PathVariable("imageCode")
            String imageCode
    );
}